#!/usr/bin/perl
#
# GuillaumePlayground.net
#
# This is free and unencumbered software released into the public domain.
# 
# Anyone is free to copy, modify, publish, use, compile, sell, or
# distribute this software, either in source code form or as a compiled
# binary, for any purpose, commercial or non-commercial, and by any
# means.
# 
# In jurisdictions that recognize copyright laws, the author or authors
# of this software dedicate any and all copyright interest in the
# software to the public domain. We make this dedication for the benefit
# of the public at large and to the detriment of our heirs and
# successors. We intend this dedication to be an overt act of
# relinquishment in perpetuity of all present and future rights to this
# software under copyright law.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.
# 
# For more information, please refer to <http://unlicense.org/>

use strict;
use LWP::UserAgent;
use Archive::Extract;
use File::Copy;

## K/V list file of FW to download
my $LIST	= "./crucial-fw.cfg";
## Put downloaded firmware here (raw zip files)
my $DWN_DIR	= "./fw";
## Temporary directory for loopback mount
my $MNT_DIR 	= "./mnt";
## Grub target file
my $GRUB_FILE	= "/etc/grub.d/45_crucial-fw";
## Grub target directory
my $ISO_DIR	= "/boot/crucial-fw";

#
# Work...
#

sub err { print STDERR join(' ', ('Err:', @_))."\n"; exit(1); }

if (! -f $LIST) {
 &err( "Need list file" );
}

# Load list
my %hfw;
open FH, $LIST;
while (<FH>) {
 chomp;
 next if (/^\s*#/ || /^\s*$/);
 my ($n,$dwl) = (/^\s*(\w+)\s+([^\s]*)/);
 if (!$n) {
  &err( "Unknown line in list file", $_ );
 }
 $hfw{$n} = $dwl;
}
close FH;

# Download
mkdir $DWN_DIR;
if (! -d $DWN_DIR) {
 &err( "Failed to create download directory", $DWN_DIR);
}

# Open target for grub
open FGB, ">$GRUB_FILE";
print FGB <<EOF;
#!/bin/sh
exec tail -n +3 \$0
submenu 'Crucial Firmware Update' {
EOF

# Get each file
my $ua;
foreach my $tg (sort {$a cmp $b} keys %hfw) {
 my $url = $hfw{$tg};
 if ($url !~ /zip$/) {
  &err( "Only manage url zip file", $url );
 }
 my ($lf) = ($url =~ '/([^/]*)$');
 my $iso = $lf; $iso =~ s/zip$/iso/;

 print "Target: $tg ($lf, $iso)\n";

 if (! -f "$DWN_DIR/$lf") {
  print STDERR "  Downloading..";
  $ua ||= LWP::UserAgent->new();
  my $req = HTTP::Request->new(GET => $url);
  my $r = $ua->request($req)->content;
  open FH, ">$DWN_DIR/$lf" or &err("$! $DWN_DIR/$lf");
  binmode FH;
  print FH $r;
  close FH;
  print STDERR " done\n";
 }

 print STDERR "  Uncompressing..";
 my $ae = Archive::Extract->new( archive => "$DWN_DIR/$lf" );
 my $ok = $ae->extract( to => $DWN_DIR );
 my $iso = $ae->files->[0];
 print STDERR " $iso";
 if (! -f "$DWN_DIR/$iso") {
  &err( "Unable to get iso associated" );
 }
 print STDERR " done\n";

 print STDERR "  Moving to $tg.iso";
 mkdir $ISO_DIR;
 move("$DWN_DIR/$iso", "$ISO_DIR/$tg.iso");
 print STDERR " done\n";

 print STDERR "  Mounting";
 mkdir $MNT_DIR;
 `mount -o loop $ISO_DIR/$tg.iso $MNT_DIR 2> /dev/null`; 
 print STDERR " done\n";

 print STDERR "  Grub -> mode";
 print FGB <<EOF;
menuentry "$tg FW" {
 insmod loopback
 set isofile="$ISO_DIR/$tg.iso"
 search -sf \$isofile
 loopback loop \$isofile
EOF

 if (-f "$MNT_DIR/boot/isolinux/memdisk" && -f "$MNT_DIR/boot/isolinux/boot2880.img") {
  print STDERR " linux16";
  print FGB <<EOF;
 linux16 (loop)/boot/isolinux/memdisk
 initrd16 (loop)/boot/isolinux/boot2880.img
EOF
  print STDERR " done\n";
 }
 elsif (-f "$MNT_DIR/boot/vmlinuz" && -f "$MNT_DIR/boot/core.gz") {
  print STDERR " initrd";
  my $append = &parse_isolinux( 'APPEND' );
  print FGB <<EOF;
 linux (loop)/boot/vmlinuz $append
 initrd (loop)/boot/core.gz
EOF
 }
 else {
  print STDERR " Unknown, failed.";
 }
 print FGB "}\n";
 print STDERR " done\n";

 print STDERR "  Unmounting";
 `umount $MNT_DIR`; 
 print STDERR " done\n";

 rmdir $MNT_DIR;
}
print FGB "}\n";
close FGB;

chmod 0755, $GRUB_FILE;

print STDERR "\n";
print STDERR "Generated in: $GRUB_FILE\n";
print STDERR "      ISO in: $ISO_DIR\n";
print STDERR "\n";
print STDERR "You need to run: 'update-grub2' and reboot to apply one FW now\n";
print STDERR "\n";


sub parse_isolinux {
 my $key = shift;
 my $out = '';
 open FH, "$MNT_DIR/boot/isolinux/isolinux.cfg";
 while (<FH>) {
  chomp;
  if (/\s*$key\s+(.*)/i) {
   $out = $1;
   last;
  }
 }
 close FH;
 return $out;
}


1;
__END__
